from zope.interface import Interface


class ISouperLayer(Interface):
    """Browser Layer for soup related views"""


class ISoupRoot(Interface):
    """Marker for root object used to persist soup mapping.
    """



import unittest
import doctest
import pprint
from interlude import interact
from plone.testing import (
    layered,
    z2,
)
from souper.plone.testing import SOUP_INTEGRATION_TESTING

optionflags = doctest.NORMALIZE_WHITESPACE | doctest.ELLIPSIS
optionflags |= doctest.REPORT_ONLY_FIRST_FAILURE

TESTFILES = [
    'locator.rst',
]


def test_suite():
    suite = unittest.TestSuite()
    suite.addTests([
            layered(
                doctest.DocFileSuite(
                    docfile,
                    globs={'interact': interact,
                           'pprint': pprint.pprint,
                           'z2': z2,
                           },
                    optionflags=optionflags,
                    ),
                layer=SOUP_INTEGRATION_TESTING,
                )
            for docfile in TESTFILES
            ])
    return suite



import transaction
from Acquisition import aq_inner
from zope.component import (
    getUtilitiesFor,
)
from souper.interfaces import (
    ICatalogFactory,
    IStorageLocator,
)
from souper.soup import (
    get_soup,
)
from Products.Five import BrowserView


class SoupAdmin(BrowserView):

    @property
    def info(self):
        return self.request.form.get('info', '')

    @property
    def existing_soups(self):
        ret = list()
        for name, soup in getUtilitiesFor(ICatalogFactory):
            ret.append(name)
        return ret

    def count(self, sid):
        soup = get_soup(sid, aq_inner(self.context))
        return len(soup.storage)

    def redirect_base(self, msg):
        url = self.context.absolute_url()
        url += '/soup-controlpanel?info=%s' % msg
        self.request.RESPONSE.redirect(url)

    def storage_path(self, sid):
        locator = IStorageLocator(self.context)
        path = locator.path(sid)
        if not path.startswith('/'):
            path = '/' + path
        return path

    def reindex_soup(self):
        sid = self.request.form.get('id')
        if not sid:
            return self.redirect_base('No soup id given!')
        soup = get_soup(sid, self.context)
        soup.reindex()
        msg = '%s reindexed.' % sid
        return self.redirect_base(msg)

    def rebuild_soup(self):
        sid = self.request.form.get('id')
        if not sid:
            return self.redirect_base('No soup id given')
        soup = get_soup(sid, self.context)
        soup.rebuild()
        msg = '%s rebuilt.' % id
        return self.redirect_base(msg)

    def move_storage(self):
        sid = self.request.form.get('id')
        if not sid:
            return self.redirect_base(u'No id given')
        path = self.request.form.get('path')
        locator = IStorageLocator(self.context)
        try:
            locator.traverse(path)
        except ValueError:
            return self.redirect_base(u'Desired path does not exist!')
        method = self.request.form.get('moveormount')
        if method == 'move':
            locator.move(sid, path)
            transaction.commit()
            return self.redirect_base(u'Moved storage of %s to %s' %
                                      (id, path))
        elif method == "mount":
            locator.set_path(sid, path)
            transaction.commit()
            return self.redirect_base(u'Mounted storage %s to %s' % (id, path))
        else:
            return self.redirect_base(u'Invalid action (move or mount only)')

    def rebuild_length(self):
        sid = self.request.form.get('id')
        if not sid:
            return self.redirect_base('No soup id given')
        soup = get_soup(sid, self.context)
        newlen = len(soup.storage.data)
        soup.storage.length.set(newlen)
        transaction.commit()
        return self.redirect_base(u'Length of storage %s is %s' % (id, newlen))



from copy import deepcopy
from Acquisition import (
    aq_inner,
    aq_parent,
)
from persistent.mapping import PersistentMapping
from zope.interface import implementer
from zope.annotation import IAnnotations
from souper.interfaces import IStorageLocator
from souper.soup import (
    SoupData,
    get_soup
)
from souper.plone.interfaces import ISoupRoot

CACHE_PREFIX = 'soup_storage_%s'
SOUPPATHS = 'SOUPPATHS'
SOUPKEY = 'SOUP-%s'


@implementer(IStorageLocator)
class StorageLocator(object):

    def __init__(self, context):
        self.context = context

    def storage(self, sid):
        """return SoupData for the given soup id
        """
        context = self.traverse(self.path(sid))
        return self.soupdata(context, sid)

    @property
    def root(self):
        """find the root in the soup, place where pathes mapping is located.
        """
        obj = aq_inner(self.context)
        while True:
            if ISoupRoot.providedBy(obj):
                return obj
            obj = aq_parent(obj)
            if not obj:
                raise AttributeError(u"Invalid soup context '%s;." %
                                     self.context)

    def path(self, sid):
        """path to object with soupdata annotations for given soup id.
        relative to root.
        """
        paths = IAnnotations(self.root).get(SOUPPATHS, {})
        return paths.get(sid, '/')

    def set_path(self, sid, newpath):
        """maps path to object with soupdata annotations for given soup id.
        it does not check if there is already a soup before nor does it warn if
        there was a soup at the old location.
        """
        self.traverse(newpath)  # check if newpath is ok
        paths = IAnnotations(self.root).get(SOUPPATHS, None)
        if paths is None:
            paths = PersistentMapping()
            IAnnotations(self.root)[SOUPPATHS] = paths
        paths[sid] = newpath
        self._invalidate_cache(sid)

    def traverse(self, path):
        """traverse to path relative to soups root and return the object there.
        """
        obj = self.root
        path = [_ for _ in path.split('/') if _]
        for name in path:
            try:
                obj = obj[name]
            except AttributeError:
                msg = u'Object at %s does not exist.' % '/'.join(path)
                raise ValueError(msg)
        return obj

    def soupdata(self, obj, sid):
        """fetches the soup data from objects annotations.
        """
        key = SOUPKEY % sid
        annotations = IAnnotations(obj)
        if not key in annotations:
            annotations[key] = SoupData()
        return annotations[key]

    def _invalidate_cache(self, sid):
        """invalidates a cache on REQUEST
        """
        # XXX unused (at the moment)
        key = CACHE_PREFIX % sid
        if self.context.REQUEST.get(key):
            del self.context.REQUEST[key]

    def move(self, sid, target_path, force=False):
        """moves soup with name ``sid`` to a other target object.
        target_path is the relative path to soup root.
        if force is true any existing soup with the same name is overwritten.
        """
        source_obj = self.traverse(self.path(sid))
        source_annotations = IAnnotations(source_obj)
        source_data = self.storage(sid)
        target_obj = self.traverse(target_path)
        target_annotations = IAnnotations(target_obj)
        datakey = SOUPKEY % sid
        if datakey in target_annotations and not force:
            raise KeyError('Annotation-Key %s already used at %s' %
                           (datakey, target_path))
        self.set_path(sid, target_path)
        target_annotations[sid] = SoupData()
        target_soup = get_soup(sid, self.context)
        for intid in source_data.data:
            target_soup.add(deepcopy(source_data.data[intid]))
        del source_annotations[datakey]
        self._invalidate_cache(sid)



from plone.testing import z2
from plone.app.testing import (
    PloneSandboxLayer,
    PLONE_INTEGRATION_TESTING,
    IntegrationTesting,
    TEST_USER_NAME,
    TEST_USER_ID,
    login,
    setRoles,
)


class SoupFixture(PloneSandboxLayer):
    defaultBases = (PLONE_INTEGRATION_TESTING,)

    def setUpZope(self, app, configurationContext):
        import zopyx.txng3.core
        import souper.plone
        self.loadZCML(package=zopyx.txng3.core)
        self.loadZCML(package=souper.plone)
        self.loadZCML(package=souper.plone.tests)

    def setUpPloneSite(self, portal):
        self.applyProfile(portal, 'souper.plone:default')
        setRoles(portal, TEST_USER_ID, ['Manager'])
        login(portal, TEST_USER_NAME)

    def tearDownZope(self, app):
        pass


SOUP_FIXTURE = SoupFixture()
SOUP_INTEGRATION_TESTING = IntegrationTesting(bases=(SOUP_FIXTURE,),
                                              name="SoupFixture:Integration")



from setuptools import setup, find_packages
import sys, os

version = "1.0-beta2"
shortdesc = \
"Plone Souper Integration: Container for many lightweight queryable Records"
longdesc = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()
longdesc += open(os.path.join(os.path.dirname(__file__), 'HISTORY.rst')).read()
longdesc += open(os.path.join(os.path.dirname(__file__), 'LICENSE.rst')).read()

setup(name='souper.plone',
      version=version,
      description=shortdesc,
      long_description=longdesc,
      classifiers=[
            'Development Status :: 5 - Production/Stable',
            'License :: OSI Approved :: BSD License',
            'Operating System :: OS Independent',
            'Programming Language :: Python',
            'Framework :: Zope2',
            'Framework :: Plone :: 4.1',
            'Framework :: Plone :: 4.2',
            'Framework :: Plone :: 4.3',
            'Intended Audience :: Developers',
            'Topic :: Software Development :: Libraries :: Python Modules'
      ], # Get strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
      keywords='container data record catalog',
      author='BlueDynamics Alliance',
      author_email='dev@bluedynamics.com',
      url='http://pypi.python.org/pypi/souper.plone',
      license='BSD',
      packages=find_packages('src'),
      package_dir={'': 'src'},
      namespace_packages=['souper'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'Plone',
          'souper',
      ],
      extras_require={
          'test': [
              'interlude',
          ],
      },
      )
